<?php
/**
 phpSQLiteAdmin is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 phpSQLiteAdmin is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with phpSQLiteAdmin; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  Copyright 2004 Richard Heyes
  Copyright 2006-2007 Lexington Warner II <lex@fosilis.com>
*/
class database
{
	var $_queryLogging;
	var $_queryLog;
	var $_errors;
	var $_callbacks;
	var $fetchMode;
	function database()
	{
		$this->_errors = array();
		$this->_queryLog = array();
		$this->_queryLogging = false;
	}
	function &factory($params)
	{
		if (empty($params['type'])) return false;
		if (empty($params['path'])) $params['path'] = dirname(__FILE__).'/';
		$objName = 'database_'.$params['type'];
		include_once($params['path'].$objName.'.php');
		return new $objName($params);
	}
	function &singleton($params)
	{
		static $db;
		if (!isset($db)) $db = database::factory($params);
		return $db;
	}
	function setFetchMode($mode = DB_FETCHMODE_ASSOC)
	{
		$this->fetchMode = $mode;
	}
	function getOne($query, $dbas = null)
	{
		if (is_object($result = &$this->query($query, $dbas)))
		{
			$result->setFetchMode(DB_FETCHMODE_NUM);
			$return = $result->fetchRow();
			$return = $return[0];
			$result->free();
		}
		else $return = false;
		return $return;
	}
	function getRow($query, $dbas = null)
	{
		if (is_object($result = &$this->query($query, $dbas)))
		{
			$return = $result->fetchRow();
			$result->free();
		}
		else $return = false;
		return $return;
	}
	function getCol($query, $column = 0, $dbas = null)
	{
		$return = array();
		$result = $this->query($query, $dbas);
		if (is_object($result))
		{
			while ($row = $result->fetchRow(DB_FETCHMODE_NUM)) $return[] = $row[(int)$column];
			return $return;
		}
	}
	function getAll($query, $dbas = null)
	{
		if (is_object($result = &$this->query($query, $dbas)))
		{
			$return = array();
			while ($row = $result->fetchRow()) $return[] = $row;
			$result->free();
			return $return;
		}
		return false;
	}
	function getAssoc($query, $dbas = null)
	{
		if (is_object($result = &$this->query($query, $dbas)))
		{
			$return = array();
			while ($row = $result->fetchRow(DB_FETCHMODE_NUM))
			{
				$key = array_shift($row);
				$value = (count($row) == 1 ? $row[0] : $row);
				$return[$key] = $value;
			}
			$result->free();
			return $return;
		}
		return false;
	}
	function addErrorHandler(&$callback)
	{
		$this->_callbacks[] = &$callback;
	}
	function throwError($errorString)
	{
		$this->_errors[] = $errorString;
		for ($i=0; $i<count($this->_callbacks); ++$i) call_user_func_array($this->_callbacks[$i], $errorString);
	}
	function getLastError()
	{
		if (!empty($this->_errors)) return $this->_errors[count($this->_errors) - 1];
		return '';
	}
	function getErrors()
	{
		return $this->_errors;
	}
	function logQuery($query, $type = 'good')
	{
		if ($this->_queryLogging) $this->_queryLog[$type][] = $query;
	}
	function enableQueryLogging()
	{
		$this->_queryLogging = true;
		register_shutdown_function(array(&$this, 'printQuerySummary'));
	}
	function disableQueryLogging()
	{
		$this->_queryLogging = false;
	}
	function printQuerySummary()
	{
		if (!empty($this->_queryLog['good'])) printf('<br /><table border="0" style="width: 100%%; background-color: #00c000; border: 1px dashed #000000"><tr><td>%s</td></tr></table><br />', implode('</td></tr><tr><td>', $this->_queryLog['good']));
		if (!empty($this->_queryLog['bad'])) printf('<br /><table border="0" style="width: 100%%; background-color: #F68500; border: 1px dashed #000000"><tr><td>%s</td></tr></table>', implode('</td></tr><tr><td>', $this->_queryLog['bad']));
	}
}
?>