<?php
/**
 phpSQLiteAdmin is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 phpSQLiteAdmin is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with phpSQLiteAdmin; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  Copyright 2004 Richard Heyes
  Copyright 2006-2007 Lexington Warner II <lex@fosilis.com>
*/
define('DB_FETCHMODE_ASSOC', SQLITE_ASSOC, true);
define('DB_FETCHMODE_NUM', SQLITE_NUM, true);
define('DB_FETCHMODE_BOTH', SQLITE_BOTH, true);
class database_sqlite extends database
{
	var $dbas;
	var $dbHandle;
	function database_sqlite($params)
	{
		$this->database();
		$this->dbas = $params['dbas'];
		$this->connect($params['dbas']);
		$this->setFetchMode(DB_FETCHMODE_ASSOC);
	}
	function connect($dbas)
	{
		$this->dbHandle = sqlite_open($dbas, 0666, $errorString);
		if (!empty($errorString)) $this->throwError($errorString);
		return is_resource($this->dbHandle);
	}
	function isConnected()
	{
		return is_resource($this->dbHandle);
	}
	function setDatabase($dbas)
	{
		if ($this->connect($dbas))
		{
			$this->dbas = $dbas;
			return true;
		}
		return false;
	}
	function quote($string)
	{
		return "'".sqlite_escape_string($string)."'";
	}
	function query($query, $dbas = null, $unbuffered = null)
	{
		if (!is_null($unbuffered) AND $unbuffered) $queryFunction = 'sqlite_unbuffered_query';
		else $queryFunction = 'sqlite_query';
		if (!is_null($dbas)) $this->setDatabase($dbas);
		if ($result = call_user_func($queryFunction, $query, $this->dbHandle) AND preg_match('/^(select|explain|pragma)/i', $query))
		{
			$return = new database_sqlite_result($result);
			$return->setFetchMode($this->fetchMode);
		}
		else $return = (bool)$result;
		$this->logQuery($query, $result ? 'good' : 'bad');
		if (!is_null($dbas)) $this->setDatabase($this->dbas);
		$this->checkForError();
		return $return;
	}
	function uQuery($query, $dbas = null)
	{
		return $this->query($query, $dbas, true);
	}
	function checkForError()
	{
		if ($errorCode = sqlite_last_error($this->dbHandle))
		{
			$this->throwError(sqlite_error_string($errorCode));
			return true;
		}
		return false;
	}
	function getEncoding()
	{
		return sqlite_libencoding();
	}
	function getVersion()
	{
		return sqlite_libversion();
	}
}
class database_sqlite_result
{
	var $result;
	var $fetchMode;
	var $currentRow;
	function database_sqlite_result(&$result)
	{
		$this->result = &$result;
		$this->fetchMode = DB_FETCHMODE_ASSOC;
	}
	function setFetchMode($mode = DB_FETCHMODE_ASSOC)
	{
		$oldmode = $this->fetchMode;
		$this->fetchMode = $mode;
		return $oldmode;
	}
	function fetchRow($mode = null)
	{
		$mode = !empty($mode) ? $mode : $this->fetchMode;
		$this->currentRow = sqlite_fetch_array($this->result, $mode);
		return $this->currentRow;
	}
	function numRows()
	{
		return sqlite_num_rows($this->result);
	}
	function free()
	{
		return false;
	}
}
?>