<?php
// +-----------------------------------------------------------------------+ 
// | Copyright (c) 2002-2003 Richard Heyes                                     | 
// | All rights reserved.                                                  | 
// |                                                                       | 
// | Redistribution and use in source and binary forms, with or without    | 
// | modification, are permitted provided that the following conditions    | 
// | are met:                                                              | 
// |                                                                       | 
// | o Redistributions of source code must retain the above copyright      | 
// |   notice, this list of conditions and the following disclaimer.       | 
// | o Redistributions in binary form must reproduce the above copyright   | 
// |   notice, this list of conditions and the following disclaimer in the | 
// |   documentation and/or other materials provided with the distribution.| 
// | o The names of the authors may not be used to endorse or promote      | 
// |   products derived from this software without specific prior written  | 
// |   permission.                                                         | 
// |                                                                       | 
// | THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   | 
// | "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     | 
// | LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR | 
// | A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT  | 
// | OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, | 
// | SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT      | 
// | LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, | 
// | DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY | 
// | THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   | 
// | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE | 
// | OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  | 
// |                                                                       | 
// +-----------------------------------------------------------------------+ 
// | Author: Richard Heyes <richard@phpguru.org>                           | 
// +-----------------------------------------------------------------------+ 
class Console_Table
{
	var $_headers;
	var $_data;
	var $_max_cols;
	var $_max_rows;
	var $_cell_lengths;
	var $_options;
	var $_padding;
	function Console_Table()
	{
		$this->_headers      = array();
		$this->_data         = array();
		$this->_cell_lengths = array();
		$this->_max_cols     = 0;
		$this->_max_rows     = 0;
		$this->_padding      = 1;
	}
	function setHeaders($headers)
	{
		$this->_headers = $headers;
		$this->_updateRowsCols($headers);
	}
	function addRow($row, $append = true)
	{
		$append ? $this->_data[] = array_values($row) : array_unshift($this->_data, array_values($row));
		$this->_updateRowsCols($row);
	}
	function insertRow($row, $row_id = 0)
	{
		array_splice($this->_data, $row_id, 0, array($row));
		$this->_updateRowsCols($row);
	}
	function addCol($col_data, $col_id = 0, $row_id = 0)
	{
		foreach ($col_data as $col_cell) {
			$this->_data[$row_id++][$col_id] = $col_cell;
		}
		$this->_updateRowsCols();
		$this->_max_cols = max($this->_max_cols, $col_id + 1);
	}
	function addData($data, $col_id = 0, $row_id = 0)
	{
		foreach ($data as $row) {
			$starting_col = $col_id;
			foreach ($row as $cell) {
				$this->_data[$row_id][$starting_col++] = $cell;
			}
			$this->_updateRowsCols();
			$this->_max_cols = max($this->_max_cols, $starting_col);
			$row_id++;
		}
	}
	function getTable()
	{
		$this->_validateTable();
		return $this->_buildTable();
	}
	function _validateTable()
	{
		for ($i=0; $i<$this->_max_rows; $i++) {
			for ($j=0; $j<$this->_max_cols; $j++) {
				if (!isset($this->_data[$i][$j])) {
					$this->_data[$i][$j] = '';
				}
				$this->_calculateCellLengths($this->_data[$i]);
			}
			ksort($this->_data[$i]);
		}
		ksort($this->_data);
	}
	function _buildTable()
	{
		$return = array();
		$rows   = $this->_data;
		for ($i=0; $i<count($rows); $i++) {
			for ($j=0; $j<count($rows[$i]); $j++) {
				if (strlen($rows[$i][$j]) < $this->_cell_lengths[$j]) {
					$rows[$i][$j] = str_pad($rows[$i][$j], $this->_cell_lengths[$j], ' ');
				}
			}
			$row_begin    = '|' . str_repeat(' ', $this->_padding);
			$row_end      = str_repeat(' ', $this->_padding) . '|';
			$implode_char = str_repeat(' ', $this->_padding) . '|' . str_repeat(' ', $this->_padding);
			$return[] = $row_begin . implode($implode_char, $rows[$i]) . $row_end;
		}
		$return = $this->_getSeparator() . "\r\n" . implode("\n", $return) . "\r\n" . $this->_getSeparator() . "\r\n";
		if (!empty($this->_headers)) {
			$return = $this->_getHeaderLine() . "\r\n" . $return;
		}
		return $return;
	}
	function _getSeparator()
	{
		foreach ($this->_cell_lengths as $cl) {
			$return[] = str_repeat('-', $cl);
		}
		$row_begin    = '+' . str_repeat('-', $this->_padding);
		$row_end      = str_repeat('-', $this->_padding) . '+';
		$implode_char = str_repeat('-', $this->_padding) . '+' . str_repeat('-', $this->_padding);
		$return = $row_begin . implode($implode_char, $return) . $row_end;
		return $return;
	}
	function _getHeaderLine()
	{
		for ($i=0; $i<$this->_max_cols; $i++) {
			if (!isset($this->_headers[$i])) {
				$this->_headers[$i] = '';
			}
		}
		for ($i=0; $i<count($this->_headers); $i++) {
			if (strlen($this->_headers[$i]) < $this->_cell_lengths[$i]) {
				$this->_headers[$i] = str_pad($this->_headers[$i], $this->_cell_lengths[$i], ' ');
			}
		}
		$row_begin    = '|' . str_repeat(' ', $this->_padding);
		$row_end      = str_repeat(' ', $this->_padding) . '|';
		$implode_char = str_repeat(' ', $this->_padding) . '|' . str_repeat(' ', $this->_padding);
		$return[] = $this->_getSeparator();
		$return[] = $row_begin . implode($implode_char, $this->_headers) . $row_end;
		return implode("\r\n", $return);
	}
	function _updateRowsCols($rowdata = null)
	{
		$this->_max_cols = max($this->_max_cols, count($rowdata));
		ksort($this->_data);
		$this->_max_rows = end(array_keys($this->_data)) + 1;
	}
	function _calculateCellLengths($row)
	{
		for ($i=0; $i<count($row); $i++) {
			$this->_cell_lengths[$i] = max(strlen(@$this->_headers[$i]), @$this->_cell_lengths[$i], strlen(@$row[$i]));
		}
	}
}
?>