<?php
/**
 phpSQLiteAdmin is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 phpSQLiteAdmin is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with phpSQLiteAdmin; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  Copyright 2004 Richard Heyes
  Copyright 2006-2007 Lexington Warner II <lex@fosilis.com>
*/
class sqlite
{
	function getDatabaseList()
	{
		global $config;
		$databases = array();
		if (!empty($config['databases']['database']))
		{
			foreach ($config['databases']['database'] as $db)
			{
				$exists = (int)file_exists($db['path']);
				$writeable = (int)($exists AND is_writeable($db['path']));
				$databases[] = array('path' => $db['path'], 'name' => $db['name'], 'exists' => $exists, 'writeable' => $writeable);
			}
		}
		return $databases;
	}
	function getTableList()
	{
		$DB = &database::singleton($_SESSION['database']['params']);
		$tables = $DB->getAll("SELECT * FROM sqlite_master WHERE type = 'table'");
		foreach ($tables as $k => $v) $tables[$k]['records'] = $DB->getOne("SELECT COUNT(*) FROM $v[name]");
		return $tables;
	}
	function getViewList()
	{
		$DB = &database::singleton($_SESSION['database']['params']);
		$views = $DB->getAll("SELECT * FROM sqlite_master WHERE type = 'view'");
		return $views;
	}
	function getIndexList($table, $getExtendedInfo = false)
	{
		$DB = &database::singleton($_SESSION['database']['params']);
		$table = sqlite::sanitiseIdentifier($table, true);
		$indexes = $DB->getAll("PRAGMA index_list($table)");
		if ($getExtendedInfo)
		{
			foreach ($indexes as $k => $index)
			{
				$indexInfo = sqlite::getIndexInfo($index['name']);
				$indexes[$k]['column'] = $indexInfo['name'];
			}
		}
		return $indexes;
	}
	function getIndexInfo($index)
	{
		$DB = &database::singleton($_SESSION['database']['params']);
		return $DB->getRow("PRAGMA index_info('$index')");
	}
	function getTableStructure($table)
	{
		$DB = &database::singleton($_SESSION['database']['params']);
		$table =  sqlite::sanitiseIdentifier($table, true);
		$structure = $DB->getAll("PRAGMA table_info($table)");
		foreach ($structure as $k => $v)
		{
			if ($v['pk'] AND strcasecmp($v['type'], 'integer') == 0) $structure[$k]['autoincrement'] = true;
			else $structure[$k]['autoincrement'] = false;
		}
		return $structure;
	}
	function getViewProperties($view)
	{
		$DB = &database::singleton($_SESSION['database']['params']);
		$view = sqlite::sanitiseIdentifier($view);
		$result = $DB->getOne("SELECT sql FROM sqlite_master WHERE name = '$view'");
		if ($result)
		{
			preg_match('/^CREATE\s+(TEMP\s+|TEMPORARY\s+)?VIEW\s+(\[?[a-z][a-z0-9]*\]?)\s+AS\s+(.+)$/i', $result, $matches);
			$return['name'] = $view;
			$return['temporary'] = !empty($matches[1]);
			$return['sql'] = $matches[3];
			return $return;
		}
		else return false;
	}
	function dropView($view)
	{
		$DB = &database::singleton($_SESSION['database']['params']);
		$view = sqlite::sanitiseIdentifier($view, true);
		return $DB->query("DROP VIEW $view");
	}
	function createView($name, $sql, $temporary = false)
	{
		$DB = &database::singleton($_SESSION['database']['params']);
		$name = sqlite::sanitiseIdentifier($name, true);
		$temporary = $temporary ? 'TEMPORARY' : '';
		$createSql = "CREATE $temporary VIEW $name AS $sql";
		return $DB->query($createSql);
	}
	function sanitiseIdentifier($input, $brackets = false)
	{
		$input = preg_replace('/[^a-z0-9]/i', '', $input);
		if ($brackets) $input = "[$input]";
		return $input;
	}
	function deleteRow($rowid, $table)
	{
		$DB = &database::singleton($_SESSION['database']['params']);
		$table = sqlite::sanitiseIdentifier($table, true);
		return (bool)$DB->query("DELETE FROM $table WHERE _ROWID_ = $rowid");
	}
	function getUserFunctions()
	{
		global $config;
		static $functionCache;
		if (!empty($functionCache)) return $functionCache;
		$DB = &database::singleton($_SESSION['database']['params']);
		$functions_pre = get_defined_functions();
		$returnList = array();
		$path = $config['base_dir'].'include/user.functions/';
		$dir = opendir($path);
		while ($filename = readdir($dir))
		{
			if (preg_match('/^sqlite_user_.+\.php$/', $filename)) include($path . $filename);
		}
		$functions_post = get_defined_functions();
		$functions = array_diff($functions_post['user'], $functions_pre['user']);
		foreach ($functions as $actual)
		{
        		if ($friendly = preg_replace('/^sqlite_user_/', '', $actual))
			{
				$returnList[] = array('actual' => $actual, 'friendly' => $friendly);
				sqlite_create_function($DB->dbHandle, $actual, $actual);
			}
		}
		$functionCache = $returnList;
		return $returnList;
	}
	function getNumArguments($function)
	{
		$functions = array('abs' => 1,
			'date' => 1,
			'datetime' => 1,
			'last_insert_rowid' => 0,
			'length' => 1,
			'lower' => 1,
			'random' => 0,
			'round' => 1,
			'soundex' => 1,
			'sqlite_version' => 0,
			'upper' => 1
		);
		if (isset($functions[$function])) return $functions[$function];
		else return null;
	}
}
?>